const url_dir = "";


eByID("bg_color").addEventListener("change", readColors);
eByID("text_color").addEventListener("change", readColors);
eByID("check_edge").addEventListener("change", readColors);
eByID("edge_color").addEventListener("change", readColors);
eByID("check_manual_time").addEventListener("change", drawAnalogClock);
eByID("analog_download").addEventListener("click", downloadAnalogClock);
document.getElementsByName("date_radio").forEach(e => e.addEventListener("change", readRadios));
document.getElementsByName("day_radio").forEach(e => e.addEventListener("change", readRadios));
document.getElementsByName("time_radio").forEach(e => e.addEventListener("change", readRadios));
document.getElementsByName("24_radio").forEach(e => e.addEventListener("change", readRadios));


const j_days = ["日","月","火","水","木","金","土"];
const e_days = ["Sun","Mon","Tue","Wed","Thu","Fri","Sat"];

var date_radio_index = 0;
var day_radio_index = 0;
var time_radio_index = 0;
var h24_radio_index = 0;

var img_base = newImage(url_dir + "images/base.png");
var img_s = newImage(url_dir + "images/s.png");
var img_m = newImage(url_dir + "images/m.png");
var img_h = newImage(url_dir + "images/h.png");
var img_c = newImage(url_dir + "images/center.png");

var year = 0;
var month = 0;
var date = 0;
var day = 0;
var hour = 0;
var minute = 0;
var second = 0;

var bg_color = "";
var text_color = "";
var edge_color = "";

window.onload = () => {
  initialize();
};

function initialize() {
  readRadios();

  function update() {
    readColors();
    updateTime();

    writeDigitalClock();
    drawAnalogClock();
  }

  setInterval(update, 25);
}



function readRadios() {
  // date
  var rs = document.getElementsByName("date_radio");
  for (var i = 0; i < rs.length; i++) {
    if (rs[i].checked) {
      date_radio_index = i; 
      break;
    }
  }

  // day
  rs = document.getElementsByName("day_radio");
  for (var i = 0; i < rs.length; i++) {
    if (rs[i].checked) {
      day_radio_index = i; 
      break;
    }
  }

  // time
  rs = document.getElementsByName("time_radio");
  for (var i = 0; i < rs.length; i++) {
    if (rs[i].checked) {
      time_radio_index = i; 
      break;
    }
  }

  // time
  rs = document.getElementsByName("24_radio");
  for (var i = 0; i < rs.length; i++) {
    if (rs[i].checked) {
      h24_radio_index = i; 
      break;
    }
  }
}


function readColors() {
  bg_color = eByID("bg_color").value;
  eByID("table").style.backgroundColor = bg_color;

  es_text = [
              eByID("date_display"), 
              eByID("day_display"), 
              eByID("time_display"), 
            ]

  text_color = eByID("text_color").value;
  edge_color = eByID("edge_color").value;

  es_text.forEach(e => {
    e.style.color = text_color;
    if (eByID("check_edge").checked) {
      e.style.letterSpacing = "2px";
      e.style.textShadow = 
            "1px 1px 0 " + edge_color + ", " + 
            "-1px 1px 0 " + edge_color + ", " + 
            "1px -1px 0 " + edge_color + ", " + 
            "-1px -1px 0 " + edge_color + ", " + 
            "0 -1px 0 " + edge_color + ", " + 
            "0 1px 0 " + edge_color + ", " + 
            "1px 0 0 " + edge_color + ", " + 
            "-1px 0 0 " + edge_color ;
    } else {
      e.style.letterSpacing = "0px";
      e.style.textShadow = "";
    }
  });
}

function updateTime() {
  var d = new Date();

  year = d.getFullYear();
  month = d.getMonth() + 1;
  date = d.getDate();
  day = d.getDay();

  if (eByID("check_manual_time").checked) {
    hour = eByID("manual_time_h").value % 24;
    minute = eByID("manual_time_m").value % 60;
    second = eByID("manual_time_s").value % 60;
  } else {
    hour = d.getHours();
    minute = d.getMinutes();
    second = d.getSeconds();
  }
}





function writeDigitalClock() {
  eByID("date_display").textContent = dateString();
  eByID("day_display").textContent = dayString();
  eByID("time_display").textContent = timeString(); 
}

function dateString() {
  var y = year;
  var m = zero_padding(month);
  var d = zero_padding(date);

  var strs =  [ "",
                y + "年" + m + "月" + d + "日",
                y + "/" + m + "/" + d,
                m + "/" + d + "/" + y,
                d + "/" + m + "/" + y
              ];

  return strs[date_radio_index];
}

function dayString() {
  var dy = day;
  var strs = [  "",
                "（" + j_days[dy] + "） ",
                " (" + e_days[dy] + ") ",
              ];
  return strs[day_radio_index];
}

function timeString() {
  var english = (time_radio_index >= 3);
  var ampm = "";
  if (hour < 12 && english)        ampm = "AM ";
  else if (hour < 12 && !english)  ampm = "午前";
  else if (hour >= 12 && english)  ampm = "PM ";
  else                             ampm = "午後";

  var hs = [  zero_padding(hour), 
              ampm + zero_padding(hour % 12), 
              zero_padding(hour % 12)
           ];
  var h = hs[h24_radio_index]

  var m = zero_padding(minute);
  var s = zero_padding(second);
  var time_strs = [
                "",
                h + "時" + m + "分" + s + "秒",
                h + "時" + m + "分",
                h + ":" + m + ":" + s,
                h + ":" + m,
              ];

  return time_strs[time_radio_index];
}





function drawAnalogClock() {
  var s = second;
  if (!eByID("second_visible").checked) s = 0;
  var m = minute + s / 60;
  var h = (hour % 12) + minute / 60 + s / 3600;

  var c = eByID("a_clock_canvas");

  clearCanvas(c);

  fillCanvas(c, "#ffffff00")

  drawImageToCanvas(c, img_base);
  if (eByID("second_visible").checked) {
    drawImageToCanvas(c, img_s, s * 6);
  }
  drawImageToCanvas(c, img_m, m * 6);
  drawImageToCanvas(c, img_h, h * 30);
  drawImageToCanvas(c, img_c);
}

function clearCanvas(canvas) {
  canvas.getContext("2d").clearRect(0, 0, canvas.width, canvas.height);
}

function drawImageToCanvas(canvas, img, angle = 0) {
  var ctx = canvas.getContext("2d");
  var w = canvas.width;
  var h = canvas.height;
  var sx = w / 2;
  var sy = h / 2;

  ctx.save();
  ctx.translate(sx, sy); 
  ctx.rotate(angle * Math.PI / 180);
  ctx.drawImage(img, -sx, -sy, w, h);
  ctx.restore();
}

function fillCanvas(canvas, color) {
  canvas.getContext("2d").fillStyle = color;
  canvas.getContext("2d").fillRect(0, 0, canvas.width, canvas.height);
}

function downloadAnalogClock() {
  var e = document.createElement("a");
  e.href = eByID("a_clock_canvas").toDataURL("image/png");

  var dl_str = "アナログ時計｜";
  dl_str += zero_padding(hour) + "時" 
  dl_str += zero_padding(minute) + "分" 
  if (eByID("second_visible").checked) {
    dl_str += zero_padding(second) + "秒";
  }
  e.download = dl_str;

  e.click();
}






function eByID(str) {
  return document.getElementById(str);
}

function newImage(s) {
  var i = new Image;
  i.src = s;
  return i;
}

function zero_padding(str) {
  return ( "00" + str ).slice( -2 );
}
